package main

import (
	"context"
	"fmt"
	"gitea.pena/PenaSide/codeword/internal/app"
	"gitea.pena/PenaSide/codeword/internal/initialize"
	"go.uber.org/zap"
	"os"
	"os/signal"
	_ "gitea.pena/PenaSide/linters-golang/pkg/dummy"
	"syscall"
	"time"
)

var (
	commit    string = os.Getenv("COMMIT")
	buildTime string = os.Getenv("BUILD_TIME")
	version   string = os.Getenv("VERSION")
)

// $env:COMMIT=$(git rev-parse --short HEAD); $env:BUILD_TIME=$(Get-Date -UFormat "%Y-%m-%dT%H:%M:%SZ"); $env:VERSION=$(git describe --tags); go run cmd/codeword/main.go

func main() {
	logger, err := zap.NewProduction()
	if err != nil {
		fmt.Printf("Failed to initialize logger: %v\n", err)
		os.Exit(1)
	}

	ptime, err := time.Parse(time.RFC3339, buildTime)
	if err != nil {
		logger.Error("Error parsing build time:", zap.Error(err))
		ptime = time.Now()
	}

	config, err := initialize.LoadConfig()
	if err != nil {
		logger.Fatal("Failed to load config", zap.Error(err))
	}

	ctx, stop := signal.NotifyContext(context.Background(), syscall.SIGINT, syscall.SIGTERM)
	defer stop()

	if err = app.Run(ctx, *config, logger, app.Build{
		Commit:    commit,
		Version:   version,
		BuildTime: ptime.Unix(),
	}); err != nil {
		logger.Fatal("App exited with error", zap.Error(err))
	}
}
