package purge_worker

import (
	"context"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/mongo"
	"go.uber.org/zap"
	"time"
)

type Deps struct {
	Logger *zap.Logger
	Mongo  *mongo.Collection
}

type PurgeWorker struct {
	logger *zap.Logger
	mongo  *mongo.Collection
}

func NewPurgeWC(deps Deps) *PurgeWorker {
	return &PurgeWorker{
		logger: deps.Logger,
		mongo:  deps.Mongo,
	}
}

func (wc *PurgeWorker) Start(ctx context.Context) {
	ticker := time.NewTicker(1 * time.Hour)
	defer ticker.Stop()

	for {
		select {
		case <-ticker.C:
			wc.processTasks(ctx)

		case <-ctx.Done():
			return
		}
	}
}

func (wc *PurgeWorker) processTasks(ctx context.Context) {
	wc.logger.Info("Checking cleaning records")

	oneHourAgo := time.Now().Add(-1 * time.Hour)

	filter := bson.M{"created_at": bson.M{"$lt": oneHourAgo}}

	result, err := wc.mongo.DeleteMany(ctx, filter)
	if err != nil {
		wc.logger.Error("Error when trying to delete old entries", zap.Error(err))
	} else {
		wc.logger.Info("Deleted documents", zap.Int64("count", result.DeletedCount))
	}
}

func (wc *PurgeWorker) Stop(ctx context.Context) error {
	return nil
}
